# Generated by Django 3.2.6 on 2021-09-21 09:41

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_cisco_support', '0005_auto_20210914_1344'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ciscosupport',
            name='coverage_end_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ciscosupport',
            name='warranty_end_date',
            field=models.DateField(blank=True, null=True),
        ),
    ]
