# Generated by Django 3.2.6 on 2021-09-14 13:44

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('netbox_cisco_support', '0004_ciscosupport_is_covered'),
    ]

    operations = [
        migrations.AlterField(
            model_name='ciscodevicetypesupport',
            name='end_of_routine_failure_analysis_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ciscodevicetypesupport',
            name='end_of_sale_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ciscodevicetypesupport',
            name='end_of_security_vul_support_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ciscodevicetypesupport',
            name='end_of_service_contract_renewal',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ciscodevicetypesupport',
            name='end_of_svc_attach_date',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ciscodevicetypesupport',
            name='end_of_sw_maintenance_releases',
            field=models.DateField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='ciscodevicetypesupport',
            name='last_date_of_support',
            field=models.DateField(blank=True, null=True),
        ),
    ]
